/*
 *     terraVox: an experimental voxel landscape engine
 *
 *     Copyright (C) 1999/2000 Alex J. Champandard
 *
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *     The tutorial that accompanies this source code can be found at
 *         http://www.flipcode.com
 *
 *     You can also visit the terraVox web site at
 *         http://atlas.cs.york.ac.uk/~ajc116/terraVox
 *
 *     And if you wish to contact me for any reason, I can be reached at
 *         alexjc@altavista.com
 *
 */


// mostly taken from Allegro, thanks to Shawn Hargreaves

#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <dpmi.h>
#include "timer.h"

static int timerInstalled = 0;
static long timerCounter = 0;            // regular pulse

static _go32_dpmi_seginfo newTimerHandler, oldTimerHandler;

/* my_timerint:
 *  Hardware level timer interrupt (int 8) handler. Calls whatever user
 *  timer routines are due for dealing with, and calculates how long until
 *  we need another timer tick.
 */
static int timerHandler()
{
   timerCounter += 1;
// acknoledge the interrupt
   outportb(0x20, 0x20);
   return 0;
}

void timerHandler_end() { }

/* install_timer:
 *  Installs the timer interrupt handler. You must do this before installing
 *  any user timer routines. You must set up the timer before trying to
 *  display a mouse pointer or using any of the GUI routines.
 */
int install_timer( int freq )
{

   if (timerInstalled) return -1;

   _go32_dpmi_lock_data( &timerCounter, sizeof(timerCounter));

   _go32_dpmi_lock_code( &timerHandler, (long)timerHandler - (long)timerHandler_end );

// make the procedure able to handle interrupts
  newTimerHandler.pm_offset = (long)timerHandler;
  _go32_dpmi_allocate_iret_wrapper(&newTimerHandler);
// backup old handler
  _go32_dpmi_get_protected_mode_interrupt_vector(0x08, &oldTimerHandler);
// set new handler
  _go32_dpmi_set_protected_mode_interrupt_vector(0x08, &newTimerHandler);

   asm volatile ("cli");

   long timeConst = 1193181/freq;
   outportb(0x43, 0x36 );
   outportb(0x40, timeConst & 0xff );
   outportb(0x40, (timeConst>>8) & 0xff );

   asm volatile ("sti");

   timerInstalled = 1;

   return 0;
}

/* remove_timer:
 *  Removes our timer handler and resets the BIOS clock. You don't normally
 *  need to call this, because allegro_exit() will do it for you.
 */
void remove_timer()
{
   if (!timerInstalled)
      return;

//   useRetrace = 0;

   asm volatile ("cli");

   outportb(0x43, 0x3c );
   outportb(0x40, 0xff );
   outportb(0x40, 0xff );

// restore old handler
  _go32_dpmi_set_protected_mode_interrupt_vector(0x08, &oldTimerHandler);
// free code segment
  _go32_dpmi_free_iret_wrapper(&newTimerHandler);

   asm volatile ("sti");

   timerInstalled = 0;
}

TIMER::TIMER( int freq )
{
  install_timer( freq );
}

TIMER::~TIMER()
{
  remove_timer();
}

long TIMER::getCount()
{
   return timerCounter;
}
